@@ -553,10 +553,15 @@ static void play () {
         fprintf (stderr, "couldn't create audio sink for %s\n", ao_driver);
         goto fail;
     }
-    if (vo_driver && !strncmp (vo_driver, "xv", 2))
+    if (vo_driver && !strncmp (vo_driver, "xv", 2)) {
         videosink = gst_element_factory_make ("xvimagesink", "videosink");
-    else
+        if (videosink) {
+            g_object_set (videosink, "autopaint-colorkey", 0, NULL);
+            g_object_set (videosink, "colorkey", 0x080810, NULL);
+        }
+    } else {
         videosink = gst_element_factory_make ("ximagesink", "videosink");
+    }
     if (!videosink)
         goto fail;
 
