@@ -232,6 +232,20 @@ void app_show_controls (Application *app)
         if (app->controls_timer)
             g_source_remove (app->controls_timer);
         app->controls_timer = g_timeout_add (2000, (GSourceFunc) _controlsTimeout, app);
+        static gboolean color_set;
+        if (!color_set) {
+            color_set = TRUE;
+            GdkColormap *cmap = gdk_colormap_get_system ();
+            GdkColor *color = g_new (GdkColor, 1);
+            if (gdk_color_parse ("#080810", color) &&
+                    (gdk_colormap_alloc_color (cmap, color, 0, 0) ||
+                     gdk_colormap_alloc_color (cmap, color, 0, 1))) {
+                gtk_widget_modify_bg (app->video_stop, GTK_STATE_NORMAL, color);
+                gtk_widget_modify_bg (app->toolbar_area, GTK_STATE_NORMAL, color);
+                gtk_widget_modify_bg (app->zoom_box, GTK_STATE_NORMAL, color);
+            }
+            g_free (color);
+        }
         gtk_widget_show_all (app->video_stop);
         if (app->show_playlist) {
             gtk_widget_show (app->toolbar_area);
