@@ -849,6 +849,7 @@ void Control::stateElementChanged (Node* n, Node::State os, Node::State ns) {
 void Control::updateViewAreaSizes () {
     if (!m_process || !VIEWAREA(m_app)->videoXId ())
         return;
+    VIEWAREA(m_app)->setUseColorKey (m_process->useColorKey ());
     VIEWAREA(m_app)->resizeEvent();
 }
 
@@ -1178,6 +1179,8 @@ void cb_playlist_prev (GtkButton * button, void *p) {
     if (ctr->playlist_data->current &&
             ctr->playlist_data->current->previousSibling ())
         jump (ctr->playlist_data->current->previousSibling (), ctr); //FIXME
+    else if (ctr->process ()->playing ())
+        ctr->process ()->seek (-200, false); // 20s
 }
 
 void cb_playlist_play (GtkButton * button, void *p) {
@@ -1194,7 +1197,12 @@ void cb_playlist_play (GtkButton * button, void *p) {
 
 void cb_playlist_next (GtkButton * button, void *p) {
     Control *ctr = (Control *)p;
-    if (ctr->process ()->playing ())
+    if (!ctr->process ()->playing ())
+        return;
+    if (ctr->playlist_data->current &&
+            !ctr->playlist_data->current->nextSibling ())
+        ctr->process ()->seek (200, false); // 20s
+    else
         ctr->process ()->stop ();
 }
 
