@@ -28,6 +28,7 @@
 #ifdef KMPLAYER_XML_FORMATS
 #include "kmplayer_asx.h"
 #include "kmplayer_atom.h"
+#include "kmplayer_opml.h"
 #include "kmplayer_rp.h"
 #include "kmplayer_rss.h"
 #include "kmplayer_smil.h"
@@ -63,6 +64,8 @@ Node * KMPlayer::fromXMLDocumentTag (NodePtr & d, const String & tag) {
         return new ATOM::Feed (d);
     else if (!strcasecmp (name, "playlist"))
         return new XSPF::Playlist (d);
+    else if (!strcasecmp (name, "opml"))
+        return new OPML::Opml (d);
     else
 #endif
     if (!strcasecmp (name, "url"))
