@@ -43,7 +43,9 @@ Process::Process (ProcessNotify * ctrl, const char * n, bool xv)
  : m_notify (ctrl), m_mrl_notify (NULL), m_viewer (0L), m_name (n),
       m_state (NotRunning), m_old_state (NotRunning),
       start_timer (0), m_repeat (0),
-      no_video (false), has_xv (xv), is_local_file (false)
+      no_video (false), has_xv (xv),
+      is_local_file (false),
+      use_colorkey (false)
       {}
 
 Process::~Process () {
@@ -605,6 +607,8 @@ KMPLAYER_NO_EXPORT bool MPlayer::seek (int pos, bool absolute) {
     }
     if (absolute && pos < 0)
         pos = 0;
+    if (pos < m_length)
+        m_position = pos;
     request_seek = pos;
     char buf[32];
     snprintf (buf, sizeof (buf), "seek %d %d", pos/10, absolute ? 2 : 0);
@@ -886,6 +890,7 @@ MPlayer *GstreamerMPlayerProcess::createGstreamerPlayer (ProcessNotify *notify)
     gst->setVideoDriver ("xv");
     gst->setAutoSubTitles (true);
     gst->setSupportsPanzoom (true);
+    gst->setUseColorKey (true);
     return gst;
 }
 
@@ -894,6 +899,7 @@ MPlayer *GstreamerMPlayerProcess::createMPlayer (ProcessNotify *notify)
     MPlayer *mp = new MPlayer (notify, "mplayer");
     mp->setAudioDriver ("pulse");
     mp->setVideoDriver ("xv:ck-method=auto");
+    //mp->setVideoDriver ("xv:ck-method=bg:ck=set -colorkey 2114 -v");
     return mp;
 }
 
@@ -915,6 +921,7 @@ bool GstreamerMPlayerProcess::ready (unsigned int xid)
 {
     process = gstreamer;
     m_viewer = xid;
+    use_colorkey = process->useColorKey();
     return process->ready (xid);
 }
 
@@ -927,6 +934,7 @@ bool GstreamerMPlayerProcess::play (const String &url)
         process->setMrlBackEndNotify (this);
         process->setRepeat (m_repeat);
         process->setHasVideo (!no_video);
+        use_colorkey = process->useColorKey();
     }
     return process->play (url);
 }
@@ -983,6 +991,7 @@ void GstreamerMPlayerProcess::stateChanged (Process *p,
             process->setRepeat (m_repeat);
             process->setHasVideo (!no_video);
             process->play (m_url);
+            use_colorkey = process->useColorKey();
             return;
         }
     } else if (ns == Process::Buffering) {
