@@ -87,6 +87,8 @@ public:// slots:
     bool hasXv () const { return has_xv; }
     void setUseXv (bool b) { has_xv = b; }
     bool isLocalFile () const { return is_local_file; }
+    void setUseColorKey (bool b) { use_colorkey = b; }
+    bool useColorKey () const { return use_colorkey; }
 protected:
     void setPosition (int p);
     //Source * m_source;
@@ -109,6 +111,7 @@ protected:
     bool no_video;
     bool has_xv;
     bool is_local_file;
+    bool use_colorkey;
     //int m_request_seek;
     //const char ** m_supported_sources;
 };
