@@ -50,6 +50,7 @@ static NPNetscapeFuncs ns_funcs;
 static NPPluginFuncs np_funcs;       /* plugin functions              */
 static NPP npp;                      /* single instance of the plugin */
 static NPWindow np_window;
+static NPSetWindowCallbackStruct ws_info;
 static NPObject *js_window;
 static NPObject *scriptable_peer;
 static NPSavedData *saved_data;
@@ -824,7 +825,6 @@ bail_out:
 
 void *nppInstanceOpen (const char *mime, uint16_t argc, char *argn[],
         char *argv[], void *ndata, NSNotify *notify) {
-    NPSetWindowCallbackStruct ws_info;
     NPError np_err;
     Display *display;
     int screen;
