@@ -142,23 +142,36 @@ void ImageData::copyImage (Surface *s, const SSize &sz, cairo_surface_t *similar
             1.0 * (((c) >> 8) & 0xff) / 255,  \
             1.0 * (((c)) & 0xff) / 255,       \
             1.0 * (((c) >> 24) & 0xff) / 255)
-
-class KMPLAYER_NO_EXPORT CairoPaintVisitor : public Visitor {
+struct KMPLAYER_NO_EXPORT PaintContext
+{
+    PaintContext (const Matrix& m, const IRect& c)
+        : matrix (m)
+        , clip (c)
+        , fit (fit_default)
+        , bg_repeat (SMIL::RegionBase::BgRepeat)
+        , bg_image (NULL)
+    {}
+    Matrix matrix;
     IRect clip;
+    Fit fit;
+    SMIL::RegionBase::BackgroundRepeat bg_repeat;
+    ImageData *bg_image;
+};
+
+class KMPLAYER_NO_EXPORT CairoPaintVisitor : public Visitor, public PaintContext {
+    IRect visit_rect;
     ViewArea *view_widget;
     cairo_surface_t * cairo_surface;
     cairo_surface_t * cairo_surface_widget;
-    Matrix matrix;
     // stack vars need for transitions
     TransitionModule *cur_transition;
     cairo_pattern_t * cur_pat;
     cairo_matrix_t cur_mat;
-    SMIL::RegionBase::BackgroundRepeat bg_repeat;
-    ImageData *bg_image;
     float opacity;
+    bool over_video;
+    bool toplevel;
 public:
     unsigned int penalty;
-    bool toplevel;
     cairo_t * cr;
 
     void traverseRegion (Node *reg, Surface *s);
@@ -166,6 +179,7 @@ public:
     void paint (TransitionModule *trans, MediaOpacity mopacity, Surface *s,
                 const IPoint &p, const IRect &);
     void video (Mrl *mt, Surface *s);
+    void addPaintRegion (const IRect& r);
 
     CairoPaintVisitor (ViewArea *v, cairo_surface_t * cs, Matrix m,
             const IRect & rect, bool toplevel=false);
@@ -192,11 +206,12 @@ public:
 KDE_NO_CDTOR_EXPORT
 CairoPaintVisitor::CairoPaintVisitor (ViewArea *v, cairo_surface_t *cs,
         Matrix m, const IRect & rect, bool top)
- : view_widget (v), clip (rect),
+ : PaintContext (m, rect), visit_rect (rect), view_widget (v),
 #if USE_IMG_SURFACE
-   cairo_surface_widget (cs), matrix (m),
-   bg_repeat (SMIL::RegionBase::BgRepeat), bg_image (NULL),
-   toplevel (top) {
+   cairo_surface_widget (cs),
+   over_video (false),
+   toplevel (top)
+{
     if (toplevel)
         cairo_surface = cairo_image_surface_create
             (CAIRO_FORMAT_RGB24, rect.width (), rect.height ());
@@ -212,9 +227,10 @@ CairoPaintVisitor::CairoPaintVisitor (ViewArea *v, cairo_surface_t *cs,
         cairo_translate (cr, -rect.x (), -rect.y ());
 #else
 # warning xrender
-   cairo_surface (cs), matrix (m),
-   bg_repeat (SMIL::RegionBase::BgRepeat), bg_image (NULL),
-   toplevel (top) {
+   cairo_surface (cs),
+   over_video (false),
+   toplevel (top)
+{
     cr = cairo_create (cs);
     if (toplevel) {
         penalty = rect.width () * rect.height () / (40 * 480);
@@ -250,6 +266,13 @@ KDE_NO_CDTOR_EXPORT CairoPaintVisitor::~CairoPaintVisitor () {
     }
 #endif
     cairo_destroy (cr);
+    if (over_video)
+        view_widget->applyPaintRegions ();
+}
+
+void CairoPaintVisitor::addPaintRegion (const IRect& r) {
+    if (over_video)
+        view_widget->addPaintRegion (r);
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (Node * n) {
@@ -307,12 +330,10 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
         IRect scr = matrix.toScreen (rect);
         if (clip.intersect (scr).isEmpty ())
             return;
-        Matrix m = matrix;
+        PaintContext ctx_save = *(PaintContext *) this;
         matrix = Matrix (rect.x(), rect.y(), s->xscale, s->yscale);
-        matrix.transform (m);
-        IRect clip_save = clip;
+        matrix.transform (ctx_save.matrix);
         clip = clip.intersect (scr);
-        SMIL::RegionBase::BackgroundRepeat bg_repeat_save = bg_repeat;
         if (SMIL::RegionBase::BgInherit != reg->bg_repeat)
             bg_repeat = reg->bg_repeat;
         cairo_save (cr);
@@ -321,11 +342,13 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
         if (!s->virtual_size.isEmpty ())
             matrix.translate (-s->x_scroll, -s->y_scroll);
 
+        if (fit_default != reg->fit)
+            fit = reg->fit;
+
         ImageActor *im = reg->media_info
              ? (ImageActor *) reg->media_info->media
              : NULL;
 
-        ImageData *bg_save = bg_image;
         ImageData *bg_img = im && !im->isEmpty() ? im->cached_img.ptr () : NULL;
         if (reg->background_image == "inherit")
             bg_img = bg_image;
@@ -390,6 +413,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
                     cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
             }
             cairo_restore (cr);
+
+            addPaintRegion (s->toScreen (s->bounds.size));
         }
         traverseRegion (reg, s);
         cs = s->firstChild ();
@@ -469,10 +494,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
             }
         }
         cairo_restore (cr);
-        bg_image = bg_save;
-        matrix = m;
-        bg_repeat = bg_repeat_save;
-        clip = clip_save;
+        *(PaintContext *) this = ctx_save;
         s->dirty = false;
     }
 }
@@ -646,6 +668,17 @@ KDE_NO_EXPORT void CairoPaintVisitor::video (Mrl *m, Surface *s) {
             s->xscale = s->yscale = 1; // either scale width/heigt or use bounds
             view_widget->setAudioVideoGeometry (s->toScreen (s->bounds.size),
                     NULL);
+            over_video = true;
+            view_widget->initPaintRegions (visit_rect);
+            IRect scr = matrix.toScreen (s->bounds);
+            IRect clip_rect = clip.intersect (scr);
+            if (clip_rect.isEmpty ())
+                return;
+            cairo_save (cr);
+            cairo_set_source_rgb (cr, 8.0 / 255, 8.0 / 255, 16.0 / 255);
+            cairo_rectangle (cr, clip_rect.x(), clip_rect.y(), clip_rect.width(), clip_rect.height());
+            cairo_fill (cr);
+            cairo_restore (cr);
         } else {
             view_widget->setAudioVideoGeometry (IRect (-60, -60, 50, 50), NULL);
         }
@@ -656,6 +689,12 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RefMediaType *ref) {
     if (!ref->media_info)
         return;
     Surface *s = ref->surface ();
+    if (fit_default != fit
+            && fit_default == ref->fit
+            && fit != ref->effective_fit) {
+        ref->effective_fit = fit;
+        s->resize (ref->calculateBounds(), false);
+    }
     if (s && ref->external_tree) {
         updateExternal (ref, s);
         return;
@@ -728,6 +767,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::paint (TransitionModule *trans,
         cairo_set_operator (cr, op);
     cairo_pattern_destroy (cur_pat);
     cairo_restore (cr);
+
+    addPaintRegion (s->toScreen (s->bounds.size));
 }
 
 static Mrl *findActiveMrl (Node *n, bool *rp_or_smil) {
@@ -932,6 +973,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
             cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
         s->dirty = false;
         cairo_restore (cr);
+
+        addPaintRegion (s->toScreen (s->bounds.size));
     }
 }
 
@@ -1748,9 +1791,7 @@ public:
         }
         destroyBackingStore ();
     }
-    void resizeSurface (Surface *s) {
-        int w = m_view_area->width ();
-        int h = m_view_area->height ();
+    void resizeSurface (Surface *s, int w, int h) {
         if (s->surface && (w != width || h != height)) {
                 //cairo_xlib_surface_set_size (s->surface, w, h);
             clearSurface (s);
@@ -1821,6 +1862,8 @@ KDE_NO_CDTOR_EXPORT ViewArea::ViewArea (Application *app)
    widget (NULL),
    socket (NULL),
    video_input (NULL),
+   video_region (NULL),
+   tmp_region (NULL),
    surface (new Surface (this)),
    last_repaint_time (0),
    next_repaint_time (0),
@@ -1831,7 +1874,8 @@ KDE_NO_CDTOR_EXPORT ViewArea::ViewArea (Application *app)
    m_event_filter_on (false),
    m_updaters_enabled (true),
    m_video_filter (true),
-   m_use_aspects (true) {
+   m_use_aspects (true),
+   m_use_colorkey (false) {
 }
 
 KDE_NO_CDTOR_EXPORT ViewArea::~ViewArea () {
@@ -1901,11 +1945,10 @@ KDE_NO_EXPORT void ViewArea::paintEvent (const IRect & rect) {
     scheduleRepaint (rect);
 }
 
-KDE_NO_EXPORT void ViewArea::updateSurfaceBounds () {
-    Single w = width (), h = height ();
+KDE_NO_EXPORT void ViewArea::updateSurfaceBounds (int w, int h) {
+    d->resizeSurface (surface.ptr (), w, h);
+    surface->resize (SRect (0, 0, w, h));
     if (surface->node) {
-        d->resizeSurface (surface.ptr ());
-        surface->resize (SRect (0, 0, w, h));
         surface->node->message (MsgSurfaceBoundsUpdate, (void *) true);
     }
     scheduleRepaint (IRect (0, 0, w, h));
@@ -1914,12 +1957,12 @@ KDE_NO_EXPORT void ViewArea::updateSurfaceBounds () {
 KDE_NO_EXPORT void ViewArea::resizeEvent () {
     if (!widget)
         return;
-    Single x, y, w = width (), h = height ();
+    Single w = width (), h = height ();
     if (w <= 0 || h <= 0)
         return;
-    updateSurfaceBounds ();
+    updateSurfaceBounds (w, h);
     if (video_node && Mrl::SingleMode == video_node->mrl ()->view_mode)
-        setAudioVideoGeometry (IRect (x, y, w, h), 0L);
+        setAudioVideoGeometry (IRect (0, 0, w, h), 0L);
 }
 
 KDE_NO_EXPORT void ViewArea::deleteEvent () {
@@ -1943,7 +1986,7 @@ void ViewArea::setAudioVideoGeometry (const IRect &rect, unsigned int * bg_color
         x += (w1 - w) / 2;
         y += (h1 - h) / 2;
     }
-    Color color (bg_color ? (int)*bg_color : 0);
+    Color color (bg_color ? (int)*bg_color : 0x080810);
     IRect wrect = IRect (x, y, w, h);
     if (m_av_geometry != wrect) {
         if (m_av_geometry.width () > 0 && m_av_geometry.height () > 0)
@@ -1959,9 +2002,86 @@ void ViewArea::setAudioVideoGeometry (const IRect &rect, unsigned int * bg_color
         gdk_window_set_background (m_app->video_socket->window, (GdkColor *) color);
         gdk_window_clear (m_app->video_socket->window);
         enableVideoInputFilter (m_video_filter);
+        if (video_region) {
+            gdk_region_destroy (video_region);
+            gdk_window_shape_combine_region (m_app->video_socket->window, NULL, 0, 0);
+        }
+        if (tmp_region)
+            gdk_region_destroy (tmp_region);
+        video_region = tmp_region = NULL;
     }
 }
 
+void ViewArea::initPaintRegions (const IRect& clip) {
+    if (!m_use_colorkey || !widget || !video_region)
+        return;
+    GdkRectangle *rects;
+    gint nr;
+    gdk_region_get_rectangles (video_region, &rects, &nr);
+    GdkRegion *region = NULL;
+    for (int i = 0; i < nr; ++i) {
+        GdkRectangle *rect = rects + i;
+        if (clip.intersect (
+                    IRect (m_av_geometry.point.x + rect->x,
+                           m_av_geometry.point.y + rect->y,
+                           rect->width,
+                           rect->height)).isEmpty ()) {
+            if (!region)
+                region = gdk_region_new ();
+            gdk_region_union_with_rect (region, rects + i);
+        }
+    }
+    g_free (rects);
+    if (tmp_region)
+        gdk_region_destroy (tmp_region);
+    tmp_region = region;
+}
+
+void ViewArea::addPaintRegion (const IRect& r) {
+    if (!m_use_colorkey || !widget)
+        return;
+    IRect ir = m_av_geometry.intersect (r);
+    if (ir.isEmpty ())
+        return;
+    if (!tmp_region)
+        tmp_region = gdk_region_new ();
+    GdkRectangle rect = {
+            ir.x() - m_av_geometry.point.x,
+            ir.y() - m_av_geometry.point.y,
+            ir.width(), ir.height() };
+    if (gdk_region_rect_in (tmp_region, &rect) != GDK_OVERLAP_RECTANGLE_IN)
+        gdk_region_union_with_rect (tmp_region, &rect);
+}
+
+void ViewArea::applyPaintRegions () {
+    if (!m_use_colorkey || !widget || (!video_region && !tmp_region))
+        return;
+    bool update = false;
+    if (tmp_region)
+        update = !video_region || !gdk_region_equal (video_region, tmp_region);
+
+    if (video_region && (update || !tmp_region)) {
+        gdk_window_shape_combine_region (m_app->video_socket->window, NULL, 0, 0);
+        if (!tmp_region) {
+            gdk_region_destroy (video_region);
+            video_region = NULL;
+            return;
+        }
+    }
+    if (update) {
+        GdkRectangle wrect = { 0, 0, m_av_geometry.size.width, m_av_geometry.size.width };
+        GdkRegion *full = gdk_region_rectangle (&wrect);
+        gdk_region_subtract (full, tmp_region);
+        gdk_window_shape_combine_region (m_app->video_socket->window, full, 0, 0);
+        gdk_region_destroy (full);
+    }
+    GdkRegion *tmp = video_region;
+    video_region = tmp_region;
+    if (tmp)
+        gdk_region_destroy (tmp);
+    tmp_region = NULL;
+}
+
 void ViewArea::enableVideoInputFilter (bool enable) {
     int x, y, w, h;
     m_video_filter = enable;
@@ -1982,8 +2102,10 @@ void ViewArea::enableVideoInputFilter (bool enable) {
 KDE_NO_EXPORT void ViewArea::setAudioVideoNode (NodePtr n) {
     video_node = n;
     m_av_geometry = IRect ();
-    if (!n || Mrl::WindowMode == n->mrl ()->view_mode)
+    if (!n || Mrl::WindowMode == n->mrl ()->view_mode) {
+        m_use_colorkey = false;
         setAudioVideoGeometry (IRect (-100, -100, 50, 50), 0L);
+    }
 }
 
 KDE_NO_EXPORT
@@ -2008,15 +2130,16 @@ void ViewArea::setAspect (float a) {
 }
 
 KDE_NO_EXPORT Surface *ViewArea::getSurface (Mrl *mrl) {
+    Single w = width (), h = height ();
     surface->clear ();
     surface->node = mrl;
     //m_view->viewer()->resetBackgroundColor ();
     if (mrl) {
-        updateSurfaceBounds ();
+        updateSurfaceBounds (w, h);
         return surface.ptr ();
     }
     d->clearSurface (surface.ptr ());
-    scheduleRepaint (IRect (0, 0, width (), height ()));
+    scheduleRepaint (IRect (0, 0, w, h));
     return 0L;
 }
 
