@@ -64,13 +64,17 @@ public:
     int height () const;
     GtkWidget *viewWidget () const { return widget; }
     GdkWindow *videoInputWindow () const { return video_input; }
+    void addPaintRegion (const IRect& r);
+    void applyPaintRegions ();
+    void initPaintRegions (const IRect& r);
     unsigned long videoXId () const;
     Mrl *videoMrl () { return video_node ? video_node->mrl () : NULL; }
     void setOutputWidget (Application *, GtkWidget * w);
     void toggleUseAspects () { m_use_aspects = !m_use_aspects; }
+    void setUseColorKey (bool b) { m_use_colorkey = b; }
 private:
     void syncVisual (const IRect & rect);
-    void updateSurfaceBounds ();
+    void updateSurfaceBounds (int width, int height);
     bool scheduleRepaintTimeout (unsigned int cur_time);
     ViewAreaPrivate *d;
     ConnectionList m_updaters;
@@ -81,6 +85,8 @@ private:
     GtkWidget *widget;
     GtkWidget *socket;
     GdkWindow *video_input;
+    GdkRegion *video_region;
+    GdkRegion *tmp_region;
     SurfacePtr surface;
     NodePtrW video_node;
     unsigned int last_repaint_time;
@@ -93,6 +99,7 @@ private:
     bool m_updaters_enabled;
     bool m_video_filter;
     bool m_use_aspects;
+    bool m_use_colorkey;
 };
 
 } // namespace KMPlayer
